
function init()
	global.trains = global.trains or {}
	for _,force in pairs(game.forces) do
		init_force(force)
	end
end

function init_force(force)
	if global.trains[force.name] == nil then
		global.trains[force.name] = {}
		log("surfaces: "..#game.surfaces)
		for _,surface in pairs(game.surfaces) do
			log("chunks: "..#surface.get_chunks())
			local count = 0
			for chunk in surface.get_chunks() do
				local top, left = chunk.x * 32, chunk.y * 32
				local bottom, right = top + 32, left + 32
				for _, locomotive in pairs(surface.find_entities_filtered{area = {{top, left}, {bottom, right}}, type = "locomotive"}) do
					global.trains[force.name][#global.trains[force.name] + 1] = locomotive
				end
				for _, cargo_wagon in pairs(surface.find_entities_filtered{area = {{top, left}, {bottom, right}}, type = "cargo-wagon"}) do
					global.trains[force.name][#global.trains[force.name] + 1] = cargo_wagon
				end
				count = count + 1
				if count % 100 == 0 then
					log("checked chunks: "..count)
				end
			end
		end
	end
end

script.on_init(function()
	init()
end)

script.on_configuration_changed(function(event)
	init()
end)

script.on_event(defines.events.on_force_created, function(event)
	init_force(event.force)
end)

script.on_event(defines.events.on_built_entity, function(event)
	local entity = event.created_entity
	if entity.type == "locomotive" or entity.type == "cargo-wagon" then
		table.insert(global.trains[entity.force.name],entity)
	end
end)

script.on_event(defines.events.on_tick, function(event)
	if game.tick % 60 == 0 then
		reChart()
	end
end)

function reChart()
	for k, surface in pairs(game.surfaces) do
		for forceName,v in pairs(global.trains) do
			local force = game.forces[forceName]
			for index,train in pairs(global.trains[forceName]) do
				if train.valid then

					local x1 = train.position.x
					local y1 = train.position.y
					local x2 = x1 + train.train.speed * 40 * math.sin(2 * math.pi * train.orientation)
					local y2 = y1 - train.train.speed * 40 * math.cos(2 * math.pi * train.orientation)
					
					if x2 < x1 then 
						local x = x1
						x1 = x2
						x2 = x
					end
					if y2 < y1 then
						local y = y1
						y1 = y2
						y2 = y
					end
					
					force.chart(surface,{{x1-2.5,y1-2.5},{x2+2.5,y2+2.5}})
				else
					table.remove(global.trains[forceName],index)
				end
			end
		end
	end
end

script.on_event(defines.events.on_sector_scanned, function(event)
	if event.radar.name == "train-tracker" then
		
	end
end)
